package fr.atlantique.imt.inf211.jobmngt.controller;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import fr.atlantique.imt.inf211.jobmngt.dao.ApplicationDao;
import fr.atlantique.imt.inf211.jobmngt.entity.Application;

@RestController
@RequestMapping("/applications")
public class ApplicationController {

    @Autowired
    private ApplicationDao applicationDao;

    /**
     * Créer une nouvelle candidature
     * @param application L'objet Application envoyé en JSON
     * @return Message de confirmation ou erreur
     */
    @Transactional
    @PostMapping("/create")
    public String createApplication(@RequestBody Application application) {
        try {
            if (application.getCandidate() == null || application.getCandidate().getId() == 0) {
                return "Erreur : le candidat est invalide.";
            }
            applicationDao.persist(application);
            System.out.println("✅ Candidature créée pour le candidat ID : " + application.getCandidate().getId());
            return "Candidature créée avec succès.";
        } catch (Exception e) {
            System.err.println("❌ Erreur lors de la création de la candidature : " + e.getMessage());
            return "Erreur lors de la création de la candidature.";
        }
    }

    /**
     * Récupérer la liste de toutes les candidatures
     * @return Liste des candidatures en JSON
     */
    @Transactional(readOnly = true)
    @GetMapping("/list")
    public List<Application> listApplications() {
        List<Application> applications = applicationDao.findAll("id", "asc");
        System.out.println("📌 Nombre de candidatures trouvées : " + applications.size());
        return applications;
    }

    /**
     * Trouver une candidature par son ID
     * @param id L'ID de la candidature
     * @return L'objet Application correspondant
     */
    @Transactional(readOnly = true)
    @GetMapping("/find/{id}")
    public Optional<Application> getApplicationById(@PathVariable int id) {
        Application application = applicationDao.findById(id);
        if (application != null) {
            System.out.println("📌 Candidature trouvée : " + application.getId());
            return Optional.of(application);
        } else {
            System.out.println("❌ Aucune candidature trouvée avec l'ID : " + id);
            return Optional.empty();
        }
    }

    /**
     * Supprimer une candidature par ID
     * @param id L'ID de la candidature à supprimer
     * @return Message de confirmation ou d'erreur
     */
    @Transactional
    @DeleteMapping("/delete/{id}")
    public String deleteApplication(@PathVariable int id) {
        Application application = applicationDao.findById(id);
        if (application != null) {
            applicationDao.remove(application);
            System.out.println("🗑️ Candidature supprimée : " + id);
            return "Candidature supprimée avec succès.";
        } else {
            return "❌ Erreur : Candidature non trouvée.";
        }
    }
}
