package fr.atlantique.imt.inf211.jobmngt.controller;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import fr.atlantique.imt.inf211.jobmngt.dao.CompanyDao;
import fr.atlantique.imt.inf211.jobmngt.dao.JobOfferDao;
import fr.atlantique.imt.inf211.jobmngt.dao.QualificationLevelDao;
import fr.atlantique.imt.inf211.jobmngt.entity.Company;
import fr.atlantique.imt.inf211.jobmngt.entity.JobOffer;
import fr.atlantique.imt.inf211.jobmngt.entity.QualificationLevel;

@RestController
@RequestMapping("/joboffers")
public class JobOfferController {

    @Autowired
    private JobOfferDao jobOfferDao;
    
    @Autowired
    private CompanyDao companyDao;

    @Autowired
    private QualificationLevelDao qualificationLevelDao;

    /**
     * Ajouter une offre d'emploi
     * @param jobOffer Offre d'emploi envoyée en JSON
     * @return Message de confirmation ou d'erreur
     */
    @Transactional
    @PostMapping("/create")
    public String addJobOffer(@RequestBody JobOffer jobOffer) {
        try {
            // Vérification de l'existence de l'entreprise associée
            Company company = companyDao.findById(jobOffer.getCompany().getId());
            if (company == null) {
                return "❌ Erreur : Entreprise introuvable avec ID " + jobOffer.getCompany().getId();
            }
            jobOffer.setCompany(company);

            // Vérification du niveau de qualification
            QualificationLevel qualificationLevel = qualificationLevelDao.findById(jobOffer.getQualificationLevel().getId());
            if (qualificationLevel == null) {
                return "❌ Erreur : Niveau de qualification introuvable avec ID " + jobOffer.getQualificationLevel().getId();
            }
            jobOffer.setQualificationLevel(qualificationLevel);

            // Persistance de l'offre
            jobOfferDao.persist(jobOffer);
            System.out.println("✅ Offre d'emploi ajoutée : " + jobOffer.getTitle());
            return "Offre d'emploi ajoutée avec succès.";
        } catch (Exception e) {
            System.err.println("❌ Erreur lors de la création de l'offre : " + e.getMessage());
            return "Erreur lors de la création de l'offre.";
        }
    }

    /**
     * Récupérer toutes les offres d'emploi
     * @return Liste des offres d'emploi
     */
    @Transactional(readOnly = true)
    @GetMapping("/list")
    public List<JobOffer> listJobOffers() {
        List<JobOffer> jobOffers = jobOfferDao.findAll("id", "asc");
        System.out.println("📌 Nombre d'offres trouvées : " + jobOffers.size());
        return jobOffers;
    }

    /**
     * Trouver une offre d'emploi par ID
     * @param id L'ID de l'offre d'emploi
     * @return L'offre correspondante
     */
    @Transactional(readOnly = true)
    @GetMapping("/find/{id}")
    public Optional<JobOffer> getJobOfferById(@PathVariable int id) {
        JobOffer jobOffer = jobOfferDao.findById(id);
        if (jobOffer != null) {
            System.out.println("📌 Offre trouvée : " + jobOffer.getTitle());
            return Optional.of(jobOffer);
        } else {
            System.out.println("❌ Aucune offre trouvée avec l'ID : " + id);
            return Optional.empty();
        }
    }
}
