package fr.atlantique.imt.inf211.jobmngt.controller;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import fr.atlantique.imt.inf211.jobmngt.dao.AppUserDao;
import fr.atlantique.imt.inf211.jobmngt.dao.CandidateDao;
import fr.atlantique.imt.inf211.jobmngt.dao.CompanyDao;
import fr.atlantique.imt.inf211.jobmngt.entity.AppUser;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;

@Controller
public class LoginController {

    @Autowired
    private AppUserDao appUserDao;

    @Autowired
    private CandidateDao candidateDao;

    @Autowired
    private CompanyDao companyDao;

    // ✅ Méthode GET pour afficher la page de login
    @GetMapping("/login")
    public String login() {
        return "login";
    }

    // ✅ Méthode POST pour traiter la connexion
    @PostMapping("/login")
    public ModelAndView checkLog(@RequestParam("mail") String mail, @RequestParam("password") String pwd,
                                 HttpServletRequest request) {
        HttpSession session = request.getSession();

        System.out.println("🔄 Tentative de connexion : " + mail);

        // Vérifier si l'utilisateur existe
        Optional<AppUser> user = appUserDao.checkLogin(new AppUser(0, mail, pwd, ""));
        if (user.isPresent()) {
            AppUser appUser = user.get();
            System.out.println("✅ Utilisateur trouvé, ID : " + appUser.getId());

            session.setAttribute("user", appUser);
            session.setAttribute("uid", appUser.getId());

            // ✅ Vérification du type d'utilisateur (Company ou Candidate)
            String userType = getUserType(appUser.getId());
            session.setAttribute("usertype", userType);

            return new ModelAndView("redirect:/");
        } else {
            System.out.println("❌ Échec de connexion pour : " + mail);
            ModelAndView mav = new ModelAndView("login");
            mav.addObject("error", "Identifiant ou mot de passe incorrect.");
            return mav;
        }
    }

    // ✅ Méthode pour identifier si l'utilisateur est une entreprise ou un candidat
    private String getUserType(int userId) {
        if (companyDao.findById(userId) != null) {
            return "company";
        } else if (candidateDao.findById(userId) != null) {
            return "candidate";
        }
        return "unknown";
    }

    // ✅ Méthode GET pour la déconnexion
    @GetMapping("/logout")
    public String logout(HttpServletRequest request) {
        HttpSession session = request.getSession();
        session.setAttribute("uid", null);
        session.setAttribute("user", null);
        return "redirect:/";
    }
}
