package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.AppUser;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.TypedQuery;

@Repository
public class AppUserDao {

    private static final Logger logger = Logger.getLogger(AppUserDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(AppUser transientInstance) {
        logger.log(Level.INFO, "Persisting AppUser: " + transientInstance);
        entityManager.persist(transientInstance);
    }

    @Transactional
    public void remove(AppUser persistentInstance) {
        logger.log(Level.INFO, "Removing AppUser: " + persistentInstance);
        if (!entityManager.contains(persistentInstance)) {
            persistentInstance = entityManager.merge(persistentInstance);
        }
        entityManager.remove(persistentInstance);
    }

    @Transactional
    public AppUser merge(AppUser detachedInstance) {
        logger.log(Level.INFO, "Merging AppUser: " + detachedInstance);
        return entityManager.merge(detachedInstance);
    }

    @Transactional(readOnly = true)
    public AppUser findById(int id) {
        logger.log(Level.INFO, "Finding AppUser by id: " + id);
        return entityManager.find(AppUser.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(c) FROM AppUser c", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<AppUser> findAll(String sort, String order) {
        // Vérification des champs valides pour éviter les erreurs SQL
        String allowedSortFields = "id,mail,password,usertype";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Défaut
        }
        String jpql = "SELECT s FROM AppUser s ORDER BY s." + sort + ("asc".equalsIgnoreCase(order) ? " ASC" : " DESC");
        return entityManager.createQuery(jpql, AppUser.class).getResultList();
    }

    @Transactional(readOnly = true)
    public Optional<AppUser> checkLogin(AppUser user) {
        String jpql = "SELECT s FROM AppUser s WHERE s.mail = :login AND s.password = :pwd";
        TypedQuery<AppUser> query = entityManager.createQuery(jpql, AppUser.class);
        query.setParameter("login", user.getMail());
        query.setParameter("pwd", user.getPassword());

        return query.getResultStream().findFirst(); // Simplification
    }

    @Transactional(readOnly = true)
    public Optional<AppUser> findByMail(String mail) {
        String jpql = "SELECT s FROM AppUser s WHERE s.mail = :mail";
        TypedQuery<AppUser> query = entityManager.createQuery(jpql, AppUser.class);
        query.setParameter("mail", mail);

        return query.getResultStream().findFirst(); // Simplification
    }

    @Transactional(readOnly = true)
    public String getUserType(int userId) {
        boolean isCompany = entityManager.createQuery(
            "SELECT COUNT(c) > 0 FROM Company c WHERE c.id = :userId", Boolean.class)
            .setParameter("userId", userId)
            .getSingleResult();

        if (isCompany) {
            return "company";
        }

        boolean isCandidate = entityManager.createQuery(
            "SELECT COUNT(c) > 0 FROM Candidate c WHERE c.id = :userId", Boolean.class)
            .setParameter("userId", userId)
            .getSingleResult();

        return isCandidate ? "candidate" : "unknown";
    }

}
