package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.Candidate;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;

@Repository
public class CandidateDao {

    private static final Logger logger = Logger.getLogger(CandidateDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(Candidate candidate) {
        logger.log(Level.INFO, "Persisting Candidate: " + candidate.getFirstName() + " " + candidate.getLastName());
        entityManager.persist(candidate);
    }

    @Transactional
    public void remove(Candidate candidate) {
        logger.log(Level.INFO, "Removing Candidate: " + candidate.getId());
        if (!entityManager.contains(candidate)) {
            candidate = entityManager.merge(candidate);
        }
        entityManager.remove(candidate);
    }

    @Transactional
    public Candidate merge(Candidate candidate) {
        logger.log(Level.INFO, "Merging Candidate: " + candidate.getId());
        return entityManager.merge(candidate);
    }

    @Transactional(readOnly = true)
    public Candidate findById(int id) {
        logger.log(Level.INFO, "Finding Candidate by id: " + id);
        return entityManager.find(Candidate.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(c) FROM Candidate c", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<Candidate> findAll(String sort, String order) {
        // Vérification des champs valides pour éviter les erreurs SQL
        String allowedSortFields = "id,lastName,firstName";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String jpql = "SELECT c FROM Candidate c ORDER BY c." + sort + ("asc".equalsIgnoreCase(order) ? " ASC" : " DESC");
        return entityManager.createQuery(jpql, Candidate.class).getResultList();
    }
}
