package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.JobOffer;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.TypedQuery;

@Repository
public class JobOfferDao {

    private static final Logger logger = Logger.getLogger(JobOfferDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(JobOffer jobOffer) {
        logger.log(Level.INFO, "Persisting JobOffer: " + jobOffer.getTitle());
        entityManager.persist(jobOffer);
    }

    @Transactional
    public void remove(JobOffer jobOffer) {
        logger.log(Level.INFO, "Removing JobOffer with id: " + jobOffer.getId());
        if (!entityManager.contains(jobOffer)) {
            jobOffer = entityManager.merge(jobOffer);
        }
        entityManager.remove(jobOffer);
    }

    @Transactional
    public JobOffer merge(JobOffer jobOffer) {
        logger.log(Level.INFO, "Merging JobOffer with id: " + jobOffer.getId());
        return entityManager.merge(jobOffer);
    }

    @Transactional(readOnly = true)
    public JobOffer findById(int id) {
        if (id <= 0) {
            logger.log(Level.WARNING, "findById() called with invalid id: " + id);
            return null;
        }
        logger.log(Level.INFO, "Finding JobOffer with id: " + id);
        return entityManager.find(JobOffer.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(j) FROM JobOffer j", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<JobOffer> findAll(String sort, String order) {
        // Vérification des champs autorisés
        String allowedSortFields = "id,title,publicationdate";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String orderDirection = "DESC";
        if ("asc".equalsIgnoreCase(order)) {
            orderDirection = "ASC";
        }

        String jpql = "SELECT j FROM JobOffer j ORDER BY j." + sort + " " + orderDirection;
        TypedQuery<JobOffer> query = entityManager.createQuery(jpql, JobOffer.class);

        logger.log(Level.INFO, "Executing findAll() query: " + jpql);
        return query.getResultList();
    }
}
