package fr.atlantique.imt.inf211.jobmngt.entity;

import java.time.LocalDate;
import java.util.HashSet;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

@Entity
@Table(name = "joboffer", schema = "public")
@JsonIdentityInfo(generator = ObjectIdGenerators.PropertyGenerator.class, property = "id")
public class JobOffer implements java.io.Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private int id;

    @Column(nullable = false, length = 50)
    private String title;

    @Column(name = "taskdescription", columnDefinition = "TEXT")
    private String taskdescription;

    @Column(name = "publicationdate", nullable = false)
    private LocalDate publicationdate = LocalDate.now();

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "compid", nullable = false)
    private Company company;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "qualiflevelid", nullable = false)
    private QualificationLevel qualificationLevel;

    @ManyToMany
    @JoinTable(
        name = "indexjoboffersector",
        joinColumns = @JoinColumn(name = "jobofferid"),
        inverseJoinColumns = @JoinColumn(name = "sectorid")
    )
    private Set<Sector> sectors = new HashSet<>();

    public JobOffer() {}

    public JobOffer(String title, String taskdescription, Company company, QualificationLevel qualificationLevel) {
        this.title = title;
        this.taskdescription = taskdescription;
        this.publicationdate = LocalDate.now();
        this.company = company;
        this.qualificationLevel = qualificationLevel;
    }

    // Getters & Setters

    public int getId() {
        return id;
    }

    public String getTitle() {
        return title;
    }

    public String getTaskdescription() {
        return taskdescription;
    }

    public LocalDate getPublicationdate() {
        return publicationdate;
    }

    public Company getCompany() {
        return company;
    }

    public QualificationLevel getQualificationLevel() {
        return qualificationLevel;
    }

    public Set<Sector> getSectors() {
        return sectors;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setTaskdescription(String taskdescription) {
        this.taskdescription = taskdescription;
    }

    public void setPublicationdate(LocalDate publicationdate) {
        this.publicationdate = publicationdate;
    }

    public void setCompany(Company company) {
        this.company = company;
    }

    public void setQualificationLevel(QualificationLevel qualificationLevel) {
        this.qualificationLevel = qualificationLevel;
    }

    public void setSectors(Set<Sector> sectors) {
        this.sectors = sectors;
    }
}
